import JavaScriptCore
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestJSObjectRef(TestCase):
    def test_values(self):
        self.assertEqual(JavaScriptCore.kJSPropertyAttributeNone, 0)
        self.assertEqual(JavaScriptCore.kJSPropertyAttributeReadOnly, 1 << 1)
        self.assertEqual(JavaScriptCore.kJSPropertyAttributeDontEnum, 1 << 2)
        self.assertEqual(JavaScriptCore.kJSPropertyAttributeDontDelete, 1 << 3)

        self.assertEqual(JavaScriptCore.kJSClassAttributeNone, 0)
        self.assertEqual(JavaScriptCore.kJSClassAttributeNoAutomaticPrototype, 1 << 1)

    def test_simple_functions(self):
        self.assertResultHasType(
            JavaScriptCore.JSClassRetain, JavaScriptCore.JSClassRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSClassRetain, 0, JavaScriptCore.JSClassRef.__typestr__
        )

        self.assertArgHasType(
            JavaScriptCore.JSClassRelease, 0, JavaScriptCore.JSClassRef.__typestr__
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMake, JavaScriptCore.JSObjectRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMake, 0, JavaScriptCore.JSContextRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMake, 1, JavaScriptCore.JSClassRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMake, 2, objc._C_PTR + objc._C_VOID
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeArray, JavaScriptCore.JSObjectRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeArray, 0, JavaScriptCore.JSContextRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeArray,
            2,
            b"n^" + JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgSizeInArg(JavaScriptCore.JSObjectMakeArray, 2, 1)
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeArray,
            3,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeDate, JavaScriptCore.JSObjectRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeDate, 0, JavaScriptCore.JSContextRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeDate,
            2,
            b"n^" + JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgSizeInArg(JavaScriptCore.JSObjectMakeDate, 2, 1)
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeDate,
            3,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeError, JavaScriptCore.JSObjectRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeError, 0, JavaScriptCore.JSContextRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeError,
            2,
            b"n^" + JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgSizeInArg(JavaScriptCore.JSObjectMakeError, 2, 1)
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeError,
            3,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeRegExp, JavaScriptCore.JSObjectRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeRegExp,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeRegExp,
            2,
            b"n^" + JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgSizeInArg(JavaScriptCore.JSObjectMakeRegExp, 2, 1)
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeRegExp,
            3,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeFunction, JavaScriptCore.JSObjectRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeFunction,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeFunction,
            1,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeFunction,
            3,
            b"n^" + JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgSizeInArg(JavaScriptCore.JSObjectMakeFunction, 3, 2)
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeFunction,
            4,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeFunction,
            5,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeFunction,
            7,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectGetPrototype, JavaScriptCore.JSValueRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetPrototype,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetPrototype,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPrototype,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPrototype,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPrototype,
            2,
            JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(JavaScriptCore.JSObjectHasProperty, objc._C_BOOL)
        self.assertArgHasType(
            JavaScriptCore.JSObjectHasProperty,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectHasProperty,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectHasProperty,
            2,
            JavaScriptCore.JSStringRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectGetProperty, JavaScriptCore.JSValueRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetProperty,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetProperty,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetProperty,
            2,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetProperty,
            3,
            b"o^" + JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertResultHasType(JavaScriptCore.JSObjectSetProperty, objc._C_VOID)
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetProperty,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetProperty,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetProperty,
            2,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetProperty, 3, JavaScriptCore.JSValueRef.__typestr__
        )
        self.assertArgHasType(JavaScriptCore.JSObjectSetProperty, 4, objc._C_UINT)
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetProperty,
            5,
            b"o^" + JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertResultHasType(JavaScriptCore.JSObjectDeleteProperty, objc._C_BOOL)
        self.assertArgHasType(
            JavaScriptCore.JSObjectDeleteProperty,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectDeleteProperty,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectDeleteProperty,
            2,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectDeleteProperty,
            3,
            b"o^" + JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectGetPropertyAtIndex,
            JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetPropertyAtIndex,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetPropertyAtIndex,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetPropertyAtIndex, 2, objc._C_UINT
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetPropertyAtIndex,
            3,
            b"o^" + JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectSetPropertyAtIndex, objc._C_VOID
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPropertyAtIndex,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPropertyAtIndex,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPropertyAtIndex, 2, objc._C_UINT
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPropertyAtIndex,
            3,
            JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPropertyAtIndex,
            4,
            b"o^" + JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectGetPrivate, objc._C_PTR + objc._C_VOID
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetPrivate, 0, JavaScriptCore.JSObjectRef.__typestr__
        )

        self.assertResultHasType(JavaScriptCore.JSObjectSetPrivate, objc._C_BOOL)
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPrivate, 0, JavaScriptCore.JSObjectRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectSetPrivate, 1, objc._C_PTR + objc._C_VOID
        )

        self.assertResultHasType(JavaScriptCore.JSObjectIsFunction, objc._C_BOOL)
        self.assertArgHasType(
            JavaScriptCore.JSObjectIsFunction,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectIsFunction, 1, JavaScriptCore.JSObjectRef.__typestr__
        )

        self.assertResultHasType(JavaScriptCore.JSObjectIsConstructor, objc._C_BOOL)
        self.assertArgHasType(
            JavaScriptCore.JSObjectIsConstructor,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectIsConstructor,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectCopyPropertyNames,
            JavaScriptCore.JSPropertyNameArrayRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectCopyPropertyNames,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectCopyPropertyNames,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSPropertyNameArrayRetain,
            JavaScriptCore.JSPropertyNameArrayRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSPropertyNameArrayRetain,
            0,
            JavaScriptCore.JSPropertyNameArrayRef.__typestr__,
        )

        self.assertArgHasType(
            JavaScriptCore.JSPropertyNameArrayRelease,
            0,
            JavaScriptCore.JSPropertyNameArrayRef.__typestr__,
        )

        SIZE_T = objc._C_ULNG_LNG

        self.assertResultHasType(JavaScriptCore.JSPropertyNameArrayGetCount, SIZE_T)
        self.assertArgHasType(
            JavaScriptCore.JSPropertyNameArrayGetCount,
            0,
            JavaScriptCore.JSPropertyNameArrayRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSPropertyNameArrayGetNameAtIndex,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSPropertyNameArrayGetNameAtIndex,
            0,
            JavaScriptCore.JSPropertyNameArrayRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSPropertyNameArrayGetNameAtIndex, 1, SIZE_T
        )

        self.assertArgHasType(
            JavaScriptCore.JSPropertyNameAccumulatorAddName,
            0,
            JavaScriptCore.JSPropertyNameAccumulatorRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSPropertyNameAccumulatorAddName,
            1,
            JavaScriptCore.JSStringRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectCallAsFunction, JavaScriptCore.JSValueRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsFunction,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsFunction,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsFunction,
            2,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(JavaScriptCore.JSObjectCallAsFunction, 3, SIZE_T)
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsFunction,
            4,
            b"n^" + JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgSizeInArg(JavaScriptCore.JSObjectCallAsFunction, 4, 3)
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsFunction,
            5,
            b"o^" + JavaScriptCore.JSObjectRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectCallAsConstructor,
            JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsConstructor,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsConstructor,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(JavaScriptCore.JSObjectCallAsConstructor, 2, SIZE_T)
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsConstructor,
            3,
            b"n^" + JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgSizeInArg(JavaScriptCore.JSObjectCallAsConstructor, 3, 2)
        self.assertArgHasType(
            JavaScriptCore.JSObjectCallAsConstructor,
            4,
            b"o^" + JavaScriptCore.JSObjectRef.__typestr__,
        )

    def test_functions(self):

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeConstructor,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeConstructor,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeConstructor,
            1,
            JavaScriptCore.JSClassRef.__typestr__,
        )
        self.assertArgIsFunction(
            JavaScriptCore.JSObjectMakeConstructor,
            2,
            b"".join(
                [
                    JavaScriptCore.JSObjectRef.__typestr__,
                    JavaScriptCore.JSContextRef.__typestr__,
                    JavaScriptCore.JSObjectRef.__typestr__,
                    objc._C_ULNG,
                    b"n^" + JavaScriptCore.JSObjectRef.__typestr__,
                    b"o^" + JavaScriptCore.JSValueRef.__typestr__,
                ]
            ),
            True,
        )

        @objc.callbackFor(JavaScriptCore.JSObjectMakeConstructor)
        def constructor(ctx, constr, argumentCount, arguments, exception):
            return None

        self.assertArgSizeInArg(constructor, 3, 2)
        self.assertArgIsIn(constructor, 3)
        self.assertArgIsOut(constructor, 4)

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeFunctionWithCallback,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeFunctionWithCallback,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeFunctionWithCallback,
            1,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgIsFunction(
            JavaScriptCore.JSObjectMakeFunctionWithCallback,
            2,
            b"".join(
                [
                    JavaScriptCore.JSValueRef.__typestr__,
                    JavaScriptCore.JSContextRef.__typestr__,
                    JavaScriptCore.JSObjectRef.__typestr__,
                    JavaScriptCore.JSObjectRef.__typestr__,
                    objc._C_ULNG,
                    b"n^" + JavaScriptCore.JSObjectRef.__typestr__,
                    b"o^" + JavaScriptCore.JSValueRef.__typestr__,
                ]
            ),
            True,
        )

        @objc.callbackFor(JavaScriptCore.JSObjectMakeFunctionWithCallback)
        def func(ctx, function, thisObject, argumentCount, arguments, exception):
            return None

        self.assertArgSizeInArg(func, 4, 3)
        self.assertArgIsIn(func, 4)
        self.assertArgIsOut(func, 5)

    @min_os_level("10.15")
    def test_functions10_15(self):
        self.assertArgIsOut(JavaScriptCore.JSObjectMakeDeferredPromise, 1)
        self.assertArgIsOut(JavaScriptCore.JSObjectMakeDeferredPromise, 2)

        self.assertArgIsOut(JavaScriptCore.JSObjectHasPropertyForKey, 3)

        self.assertArgIsOut(JavaScriptCore.JSObjectGetPropertyForKey, 3)

        self.assertArgIsOut(JavaScriptCore.JSObjectSetPropertyForKey, 5)

        self.assertArgIsOut(JavaScriptCore.JSObjectDeletePropertyForKey, 3)

    def testUnsupported(self):
        self.assertNotHasAttr(JavaScriptCore, "kJSClassDefinitionEmpty")
        self.assertNotHasAttr(JavaScriptCore, "JSStaticValue")
        self.assertNotHasAttr(JavaScriptCore, "JSStaticFunction")
        self.assertNotHasAttr(JavaScriptCore, "JSClassDefinition")

        self.assertNotHasAttr(JavaScriptCore, "JSClassCreate")



import JavaScriptCore
from PyObjCTools.TestSupport import TestCase


class TestWebKitAvailability(TestCase):
    def test_constants(self):
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_1_0, 0x0100)
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_1_1, 0x0110)
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_1_2, 0x0120)
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_1_3, 0x0130)
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_2_0, 0x0200)
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_3_0, 0x0300)
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_3_1, 0x0310)
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_4_0, 0x0400)
        self.assertEqual(JavaScriptCore.WEBKIT_VERSION_LATEST, 0x9999)



import objc
from Cocoa import NSDocument
from loader import MHTLoader


class MHTDocument(NSDocument):
    locationbox = objc.IBOutlet()
    webview = objc.IBOutlet()

    path = None
    statusText = None

    @objc.IBAction
    def navigateHistory_(self, sender):
        if sender.selectedSegment() == 0:
            self.webview.goBack_(sender)
        else:
            self.webview.goForward_(sender)

    def windowNibName(self):
        return "MHTDocument"

    def readFromFile_ofType_(self, path, tp):
        if self.webview is None:
            self.path = path
        else:
            self.readMHT_(path)

        return True

    def writeToFile_ofType_(self, path, tp):
        # TODO: "save-as" functionality
        return False

    def windowControllerDidLoadNib_(self, controller):
        if self.path:
            self.readMHT_(self.path)

    def readMHT_(self, path):
        self.mht = MHTLoader(path)
        self.locationbox.setStringValue_(self.mht.fixupURL(self.mht.root))
        archive = self.mht.asWebArchive()
        print("Archive", archive.description())
        with open("/tmp/archive.webarchive", "wb") as fp:
            fp.write(archive.data().bytes())
        self.webview.mainFrame().stopLoading()
        self.webview.mainFrame().loadArchive_(archive)
        1 / 0



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLStyleElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLStyleElement.disabled)
        self.assertArgIsBOOL(WebKit.DOMHTMLStyleElement.setDisabled_, 0)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLTextAreaElement(TestCase):
    def testMehods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLTextAreaElement.willValidate)



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import WebKit  # noqa: F401
import objc


class TestWebDownload(TestCase):
    @min_sdk_level("10.11")
    def testProtocols(self):
        objc.protocolNamed("WebDownloadDelegate")



import JavaScriptCore
import objc
from PyObjCTools.TestSupport import TestCase


class TestJSBase(TestCase):
    def test_functions(self):
        self.assertResultHasType(
            JavaScriptCore.JSStringCreateWithCharacters,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSStringCreateWithCharacters, 0, b"n^" + objc._C_UNICHAR
        )
        self.assertArgSizeInArg(JavaScriptCore.JSStringCreateWithCharacters, 0, 1)

        self.assertResultHasType(
            JavaScriptCore.JSStringCreateWithUTF8CString,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSStringCreateWithUTF8CString,
            0,
            b"n^" + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgIsNullTerminated(JavaScriptCore.JSStringCreateWithUTF8CString, 0)

        self.assertResultHasType(
            JavaScriptCore.JSStringRetain, JavaScriptCore.JSStringRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSStringRetain, 0, JavaScriptCore.JSStringRef.__typestr__
        )

        self.assertArgHasType(
            JavaScriptCore.JSStringRelease, 0, JavaScriptCore.JSStringRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSStringGetLength, 0, JavaScriptCore.JSStringRef.__typestr__
        )

        val = JavaScriptCore.JSStringCreateWithUTF8CString(b"hello world")
        self.assertEqual(JavaScriptCore.JSStringGetLength(val), len("hello world"))
        self.assertTrue(
            JavaScriptCore.JSStringGetMaximumUTF8CStringSize(val)
            >= len("hello world") + 1
        )

        self.assertFalse(hasattr(JavaScriptCore, "JSStringGetCharactersPtr"))

        self.assertArgSizeInArg(JavaScriptCore.JSStringGetUTF8CString, 1, 2)
        self.assertArgSizeInResult(JavaScriptCore.JSStringGetUTF8CString, 1)
        self.assertArgIsOut(JavaScriptCore.JSStringGetUTF8CString, 1)

        self.assertResultHasType(JavaScriptCore.JSStringIsEqual, objc._C_BOOL)

        self.assertResultHasType(
            JavaScriptCore.JSStringIsEqualToUTF8CString, objc._C_BOOL
        )
        self.assertArgHasType(
            JavaScriptCore.JSStringIsEqualToUTF8CString, 1, b"n^" + objc._C_CHAR_AS_TEXT
        )
        self.assertArgIsNullTerminated(JavaScriptCore.JSStringIsEqualToUTF8CString, 1)



from PyObjCTools.TestSupport import TestCase
import WebKit  # noqa: F401


class TestWebPluginHelper(WebKit.NSObject):
    def webPlugInSetIsSelected_(self, v):
        pass


class TestWebPlugin(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(TestWebPluginHelper.webPlugInSetIsSelected_, 0)



import JavaScriptCore
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestJSManagedValue(TestCase):
    @min_os_level("10.9")
    def test_classes(self):
        self.assertHasAttr(JavaScriptCore, "JSManagedValue")



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestWebView(TestCase):
    def testConstants(self):
        self.assertIsInstance(WebKit.WebElementDOMNodeKey, str)
        self.assertIsInstance(WebKit.WebElementFrameKey, str)
        self.assertIsInstance(WebKit.WebElementImageAltStringKey, str)
        self.assertIsInstance(WebKit.WebElementImageKey, str)
        self.assertIsInstance(WebKit.WebElementImageRectKey, str)
        self.assertIsInstance(WebKit.WebElementImageURLKey, str)
        self.assertIsInstance(WebKit.WebElementIsSelectedKey, str)
        self.assertIsInstance(WebKit.WebElementLinkURLKey, str)
        self.assertIsInstance(WebKit.WebElementLinkTargetFrameKey, str)
        self.assertIsInstance(WebKit.WebElementLinkTitleKey, str)
        self.assertIsInstance(WebKit.WebElementLinkLabelKey, str)
        self.assertIsInstance(WebKit.WebViewProgressStartedNotification, str)
        self.assertIsInstance(WebKit.WebViewProgressEstimateChangedNotification, str)
        self.assertIsInstance(WebKit.WebViewProgressFinishedNotification, str)
        self.assertIsInstance(WebKit.WebViewDidBeginEditingNotification, str)
        self.assertIsInstance(WebKit.WebViewDidChangeNotification, str)
        self.assertIsInstance(WebKit.WebViewDidEndEditingNotification, str)
        self.assertIsInstance(WebKit.WebViewDidChangeTypingStyleNotification, str)
        self.assertIsInstance(WebKit.WebViewDidChangeSelectionNotification, str)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.WebView.canShowMIMEType_)
        self.assertResultIsBOOL(WebKit.WebView.canShowMIMETypeAsHTML_)
        self.assertResultIsBOOL(WebKit.WebView.shouldCloseWithWindow)
        self.assertArgIsBOOL(WebKit.WebView.setShouldCloseWithWindow_, 0)
        self.assertArgIsBOOL(WebKit.WebView.setMaintainsBackForwardList_, 0)
        self.assertResultIsBOOL(WebKit.WebView.goBack)
        self.assertResultIsBOOL(WebKit.WebView.goForward)
        self.assertResultIsBOOL(WebKit.WebView.goToBackForwardItem_)
        self.assertResultIsBOOL(WebKit.WebView.supportsTextEncoding)
        self.assertResultIsBOOL(WebKit.WebView.searchFor_direction_caseSensitive_wrap_)
        self.assertArgIsBOOL(WebKit.WebView.searchFor_direction_caseSensitive_wrap_, 1)
        self.assertArgIsBOOL(WebKit.WebView.searchFor_direction_caseSensitive_wrap_, 2)
        self.assertArgIsBOOL(WebKit.WebView.searchFor_direction_caseSensitive_wrap_, 3)
        self.assertResultIsBOOL(WebKit.WebView.isLoading)
        self.assertResultIsBOOL(WebKit.WebView.drawsBackground)
        self.assertArgIsBOOL(WebKit.WebView.setDrawsBackground_, 0)
        self.assertResultIsBOOL(WebKit.WebView.canGoBack)
        self.assertResultIsBOOL(WebKit.WebView.canGoForward)
        self.assertResultIsBOOL(WebKit.WebView.canMakeTextLarger)
        self.assertResultIsBOOL(WebKit.WebView.canMakeTextSmaller)
        self.assertResultIsBOOL(WebKit.WebView.canMakeTextStandardSize)
        self.assertResultIsBOOL(WebKit.WebView.maintainsInactiveSelection)
        self.assertResultIsBOOL(WebKit.WebView.isEditable)
        self.assertArgIsBOOL(WebKit.WebView.setEditable_, 0)
        self.assertResultIsBOOL(WebKit.WebView.smartInsertDeleteEnabled)
        self.assertArgIsBOOL(WebKit.WebView.setSmartInsertDeleteEnabled_, 0)
        self.assertResultIsBOOL(WebKit.WebView.isContinuousSpellCheckingEnabled)
        self.assertArgIsBOOL(WebKit.WebView.setContinuousSpellCheckingEnabled_, 0)

        self.assertArgIsBOOL(WebKit.WebView.setShouldUpdateWhileOffscreen_, 0)
        self.assertResultIsBOOL(WebKit.WebView.shouldUpdateWhileOffscreen)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKWebView(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(WebKit.WKWebView.isLoading)
        self.assertResultIsBOOL(WebKit.WKWebView.hasOnlySecureContent)
        self.assertResultIsBOOL(WebKit.WKWebView.canGoBack)
        self.assertResultIsBOOL(WebKit.WKWebView.canGoForward)
        self.assertResultIsBOOL(WebKit.WKWebView.allowsBackForwardNavigationGestures)
        self.assertArgIsBOOL(
            WebKit.WKWebView.setAllowsBackForwardNavigationGestures_, 0
        )
        self.assertResultIsBOOL(WebKit.WKWebView.allowsMagnification)
        self.assertArgIsBOOL(WebKit.WKWebView.setAllowsMagnification_, 0)
        self.assertArgIsBlock(
            WebKit.WKWebView.evaluateJavaScript_completionHandler_, 1, b"v@@"
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(WebKit.WKWebView.allowsLinkPreview)
        self.assertArgIsBOOL(WebKit.WKWebView.setAllowsLinkPreview_, 0)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(
            WebKit.WKWebView.takeSnapshotWithConfiguration_completionHandler_, 1, b"v@@"
        )
        self.assertResultIsBOOL(WebKit.WKWebView.handlesURLScheme_)

    @min_os_level("10.15.4")
    def testMethods10_15_4(self):
        self.assertArgIsBlock(
            WebKit.WKWebView.createPDFWithConfiguration_completionHandler_, 1, b"v@@"
        )
        self.assertArgIsBlock(
            WebKit.WKWebView.createWebArchiveDataWithCompletionHandler_, 0, b"v@@"
        )
        self.assertArgIsBlock(
            WebKit.WKWebView.findString_withConfiguration_completionHandler_, 2, b"v@"
        )

    @min_os_level("10.16")
    def testMethods10_16(self):
        self.assertArgIsBlock(
            WebKit.WKWebView.evaluateJavaScript_inFrame_inContentWorld_completionHandler_,
            3,
            b"v@@",
        )
        self.assertArgIsBlock(
            WebKit.WKWebView.callAsyncJavaScript_arguments_inFrame_inContentWorld_completionHandler_,
            4,
            b"v@@",
        )



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import WebKit
import objc


class TestWebEditingDelegateHelper(WebKit.NSObject):
    def webView_shouldBeginEditingInDOMRange_(self, a, b):
        return 1

    def webView_shouldEndEditingInDOMRange_(self, a, b):
        return 1

    def webView_shouldInsertNode_replacingDOMRange_givenAction_(self, a, b, c, d):
        return 1

    def webView_shouldInsertText_replacingDOMRange_givenAction_(self, a, b, c, d):
        return 1

    def webView_shouldDeleteDOMRange_(self, a, b):
        return 1

    def webView_shouldChangeSelectedDOMRange_toDOMRange_affinity_stillSelecting_(
        self, a, b, c, d, e
    ):
        return 1

    def webView_shouldApplyStyle_toElementsInDOMRange_(self, a, b, c):
        return 1

    def webView_shouldChangeTypingStyle_toStyle_(self, a, b, c):
        return 1

    def webView_doCommandBySelector_(self, a, b):
        return 1


class TestWebEditingDelegate(TestCase):
    @min_sdk_level("10.11")
    def testProtocols(self):
        objc.protocolNamed("WebEditingDelegate")

    def testConstants(self):
        self.assertEqual(WebKit.WebViewInsertActionTyped, 0)
        self.assertEqual(WebKit.WebViewInsertActionPasted, 1)
        self.assertEqual(WebKit.WebViewInsertActionDropped, 2)

    def testMethods(self):
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldBeginEditingInDOMRange_
        )
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldEndEditingInDOMRange_
        )
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldInsertNode_replacingDOMRange_givenAction_
        )
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldInsertText_replacingDOMRange_givenAction_
        )
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldDeleteDOMRange_
        )
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldChangeSelectedDOMRange_toDOMRange_affinity_stillSelecting_  # noqa: B950
        )
        self.assertArgIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldChangeSelectedDOMRange_toDOMRange_affinity_stillSelecting_,  # noqa: B950
            4,
        )
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldApplyStyle_toElementsInDOMRange_
        )
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_shouldChangeTypingStyle_toStyle_
        )
        self.assertResultIsBOOL(
            TestWebEditingDelegateHelper.webView_doCommandBySelector_
        )



import Cocoa  # noqa: F401
from PyObjCTools.TestSupport import TestCase

try:
    import Quartz  # noqa: F401
except ImportError:
    Quartz = None  # noqa: F401


class TestCarbonUtils(TestCase):
    pass



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMMouseEvent(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMMouseEvent.ctrlKey)
        self.assertResultIsBOOL(WebKit.DOMMouseEvent.shiftKey)
        self.assertResultIsBOOL(WebKit.DOMMouseEvent.altKey)
        self.assertResultIsBOOL(WebKit.DOMMouseEvent.metaKey)

        self.assertArgIsBOOL(
            WebKit.DOMMouseEvent.initMouseEvent_canBubble_cancelable_view_detail_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_button_relatedTarget_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            WebKit.DOMMouseEvent.initMouseEvent_canBubble_cancelable_view_detail_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_button_relatedTarget_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(
            WebKit.DOMMouseEvent.initMouseEvent_canBubble_cancelable_view_detail_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_button_relatedTarget_,  # noqa: B950
            9,
        )
        self.assertArgIsBOOL(
            WebKit.DOMMouseEvent.initMouseEvent_canBubble_cancelable_view_detail_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_button_relatedTarget_,  # noqa: B950
            10,
        )
        self.assertArgIsBOOL(
            WebKit.DOMMouseEvent.initMouseEvent_canBubble_cancelable_view_detail_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_button_relatedTarget_,  # noqa: B950
            11,
        )
        self.assertArgIsBOOL(
            WebKit.DOMMouseEvent.initMouseEvent_canBubble_cancelable_view_detail_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_button_relatedTarget_,  # noqa: B950
            12,
        )

        self.assertArgIsBOOL(WebKit.DOMMouseEvent.initMouseEvent_______________, 1)
        self.assertArgIsBOOL(WebKit.DOMMouseEvent.initMouseEvent_______________, 2)
        self.assertArgIsBOOL(WebKit.DOMMouseEvent.initMouseEvent_______________, 9)
        self.assertArgIsBOOL(WebKit.DOMMouseEvent.initMouseEvent_______________, 10)
        self.assertArgIsBOOL(WebKit.DOMMouseEvent.initMouseEvent_______________, 11)
        self.assertArgIsBOOL(WebKit.DOMMouseEvent.initMouseEvent_______________, 12)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKNavigationResponse(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(WebKit.WKNavigationResponse.isForMainFrame)
        self.assertResultIsBOOL(WebKit.WKNavigationResponse.canShowMIMEType)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMImplementation(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMImplementation.hasFeature_version_)
        self.assertResultIsBOOL(WebKit.DOMImplementation.hasFeature__)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestDOMHTMLPreElement(TestCase):
    @min_os_level("10.6")
    def testMehods10_6(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLPreElement.wrap)
        self.assertArgIsBOOL(WebKit.DOMHTMLPreElement.setWrap_, 0)



import JavaScriptCore
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestJSValue(TestCase):
    @min_os_level("10.9")
    def test_classes(self):
        self.assertHasAttr(JavaScriptCore, "JSValue")

        self.assertArgIsBOOL(JavaScriptCore.JSValue.valueWithBool_inContext_, 0)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.toBool)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.deleteProperty_)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.hasProperty_)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isUndefined)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isNull)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isBoolean)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isNumber)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isString)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isObject)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isEqualToObject_)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isEqualWithTypeCoercionToObject_)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isInstanceOf_)

    @min_os_level("10.11")
    def test_classes10_11(self):
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isArray)
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isDate)

    @min_os_level("10.15")
    def test_methods10_15(self):
        self.assertArgIsBlock(
            JavaScriptCore.JSValue.valueWithNewPromiseInContext_fromExecutor_, 1, b"v@@"
        )
        self.assertResultIsBOOL(JavaScriptCore.JSValue.isSymbol)

    @min_os_level("10.9")
    def test_contants(self):
        self.assertIsInstance(JavaScriptCore.JSPropertyDescriptorWritableKey, str)
        self.assertIsInstance(JavaScriptCore.JSPropertyDescriptorEnumerableKey, str)
        self.assertIsInstance(JavaScriptCore.JSPropertyDescriptorConfigurableKey, str)
        self.assertIsInstance(JavaScriptCore.JSPropertyDescriptorValueKey, str)
        self.assertIsInstance(JavaScriptCore.JSPropertyDescriptorGetKey, str)
        self.assertIsInstance(JavaScriptCore.JSPropertyDescriptorSetKey, str)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestDOMNode(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_ELEMENT_NODE, 1)
        self.assertEqual(WebKit.DOM_ATTRIBUTE_NODE, 2)
        self.assertEqual(WebKit.DOM_TEXT_NODE, 3)
        self.assertEqual(WebKit.DOM_CDATA_SECTION_NODE, 4)
        self.assertEqual(WebKit.DOM_ENTITY_REFERENCE_NODE, 5)
        self.assertEqual(WebKit.DOM_ENTITY_NODE, 6)
        self.assertEqual(WebKit.DOM_PROCESSING_INSTRUCTION_NODE, 7)
        self.assertEqual(WebKit.DOM_COMMENT_NODE, 8)
        self.assertEqual(WebKit.DOM_DOCUMENT_NODE, 9)
        self.assertEqual(WebKit.DOM_DOCUMENT_TYPE_NODE, 10)
        self.assertEqual(WebKit.DOM_DOCUMENT_FRAGMENT_NODE, 11)
        self.assertEqual(WebKit.DOM_NOTATION_NODE, 12)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(WebKit.DOM_DOCUMENT_POSITION_DISCONNECTED, 1)
        self.assertEqual(WebKit.DOM_DOCUMENT_POSITION_PRECEDING, 2)
        self.assertEqual(WebKit.DOM_DOCUMENT_POSITION_FOLLOWING, 4)
        self.assertEqual(WebKit.DOM_DOCUMENT_POSITION_CONTAINS, 8)
        self.assertEqual(WebKit.DOM_DOCUMENT_POSITION_CONTAINED_BY, 16)
        self.assertEqual(WebKit.DOM_DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC, 32)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMNode.hasChildNodes)
        self.assertArgIsBOOL(WebKit.DOMNode.cloneNode_, 0)
        self.assertResultIsBOOL(WebKit.DOMNode.isSupported_version_)
        self.assertResultIsBOOL(WebKit.DOMNode.isSupported__)
        self.assertResultIsBOOL(WebKit.DOMNode.hasAttributes)
        self.assertResultIsBOOL(WebKit.DOMNode.isSameNode_)
        self.assertResultIsBOOL(WebKit.DOMNode.isEqualNode_)

        self.assertResultIsBOOL(WebKit.DOMNode.isContentEditable)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(WebKit.DOMNode.isDefaultNamespace_)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(WebKit.DOMNode.contains_)



"""
Script for building the example.

Usage:
    python2 setup.py py2app
"""
from setuptools import setup

MIME = "application/x-pyobjc-demo-webkitinterpreter"
plist = {
    "NSPrincipalClass": "WebKitInterpreter",
    "WebPluginName": "WebKit PyInterpreter Plug-In",
    "WebPluginDescription": "PyObjC demo that embeds a Python interpreter",
    "CFBundlePackageType": "WBPL",
    "WebPluginMIMETypes": {
        "MIME": {
            "WebPluginExtensions": ["webkitinterpreter"],
            "WebPluginTypeDescription": "WebKit PyInterpreter",
        }
    },
}

setup(
    name="WebKitInterpreter",
    plugin=["WebKitInterpreter.py"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa", "pyobjc-framework-WebKit"],
)



"""
Minimal applescript support.

The PyDocEventHandler handles just the event that is used to open URLs. Thanks
to this class you can use ``open pydoc:///os.open`` from a command-line, or
add ``pydoc:///`` to HTML files.
"""
import struct

import objc
from Cocoa import NSAppleEventManager, NSObject


def fourCharToInt(code):
    return struct.unpack(">l", code)[0]


class PyDocEventHandler(NSObject):
    webview = objc.IBOutlet("webview")
    urlfield = objc.IBOutlet("urlfield")

    def handleEvent_withReplyEvent_(self, event, replyEvent):
        theURL = event.descriptorForKeyword_(fourCharToInt(b"----"))

        self.urlfield.setStringValue_(theURL.stringValue())
        self.webview.takeStringURLFrom_(theURL)

    def awakeFromNib(self):
        manager = NSAppleEventManager.sharedAppleEventManager()

        # Add a handler for the event GURL/GURL. One might think that
        # Carbon.AppleEvents.kEISInternetSuite/kAEISGetURL would work,
        # but the system headers (and hence the Python wrapper for those)
        # are wrong.
        manager.setEventHandler_andSelector_forEventClass_andEventID_(
            self,
            "handleEvent:withReplyEvent:",
            fourCharToInt(b"GURL"),
            fourCharToInt(b"GURL"),
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestDOMHTMLButtonElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLButtonElement.disabled)
        self.assertArgIsBOOL(WebKit.DOMHTMLButtonElement.setDisabled_, 0)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLButtonElement.willValidate)
        self.assertResultIsBOOL(WebKit.DOMHTMLButtonElement.autofocus)
        self.assertArgIsBOOL(WebKit.DOMHTMLButtonElement.setAutofocus_, 0)



import keyword
import sys
import time
import traceback
from code import InteractiveConsole

import objc
from Cocoa import (
    NSZeroRect,
    NSControlKeyMask,
    NSViewWidthSizable,
    NSAnyEventMask,
    NSApplication,
    NSAttributedString,
    NSBundle,
    NSColor,
    NSDate,
    NSDefaultRunLoopMode,
    NSFont,
    NSFontAttributeName,
    NSForegroundColorAttributeName,
    NSKeyDown,
    NSLog,
    NSObject,
    NSView,
    NSScrollView,
    NSViewHeightSizable,
    NSTextView,
)
from objc import NO, YES, super

try:
    from code import softspace
except ImportError:
    softspace = None

FLT_MAX = 3.402_823_47e38

try:
    unicode
except NameError:
    unicode = str

try:
    sys.ps1
except AttributeError:
    sys.ps1 = ">>> "
try:
    sys.ps2
except AttributeError:
    sys.ps2 = "... "


class PseudoUTF8Output(object):
    softspace = 0

    def __init__(self, writemethod):
        self._write = writemethod

    def write(self, s):
        if not isinstance(s, unicode):
            s = s.decode("utf-8", "replace")
        self._write(s)

    def writelines(self, lines):
        for line in lines:
            self.write(line)

    def flush(self):
        pass

    def isatty(self):
        return True


class PseudoUTF8Input(object):
    softspace = 0

    def __init__(self, readlinemethod):
        self._buffer = ""
        self._readline = readlinemethod

    def read(self, chars=None):
        if chars is None:
            if self._buffer:
                rval = self._buffer
                self._buffer = ""
                if rval.endswith("\r"):
                    rval = rval[:-1] + "\n"
                return rval.encode("utf-8")
            else:
                return self._readline("\x04")[:-1].encode("utf-8")

        else:
            while len(self._buffer) < chars:
                self._buffer += self._readline("\x04\r")
                if self._buffer.endswith("\x04"):
                    self._buffer = self._buffer[:-1]
                    break
            rval, self._buffer = self._buffer[:chars], self._buffer[chars:]
            return rval.encode("utf-8").replace("\r", "\n")

    def readline(self):
        if "\r" not in self._buffer:
            self._buffer += self._readline("\x04\r")

        if self._buffer.endswith("\x04"):
            rval = self._buffer[:-1].encode("utf-8")

        elif self._buffer.endswith("\r"):
            rval = self._buffer[:-1].encode("utf-8") + "\n"

        self._buffer = ""

        return rval


class AsyncInteractiveConsole(InteractiveConsole):
    lock = False
    buffer = None

    def __init__(self, *args, **kwargs):
        InteractiveConsole.__init__(self, *args, **kwargs)
        self.locals["__interpreter__"] = self

    def asyncinteract(self, write=None, banner=None):
        if self.lock:
            raise ValueError("Can't nest")
        self.lock = True
        if write is None:
            write = self.write
        cprt = 'Type "help", "copyright", "credits" or "license" for more information.'
        if banner is None:
            write(
                "Python %s in %s\n%s\n"
                % (
                    sys.version,
                    NSBundle.mainBundle().objectForInfoDictionaryKey_("CFBundleName"),
                    cprt,
                )
            )
        else:
            write(banner + "\n")

        more = 0
        _buff = []
        try:
            while True:
                if more:
                    prompt = sys.ps2
                else:
                    prompt = sys.ps1
                write(prompt)
                # yield the kind of prompt we have
                yield more
                # next input function
                yield _buff.append
                more = self.push(_buff.pop())
        except:  # noqa: E722, B001
            self.lock = False
            raise
        self.lock = False

    def resetbuffer(self):
        self.lastbuffer = self.buffer
        InteractiveConsole.resetbuffer(self)

    def runcode(self, code):
        try:
            exec(code, self.locals)
        except SystemExit:
            raise
        except:  # noqa: E722, B001
            self.showtraceback()
        else:
            if softspace is not None and softspace(sys.stdout, 0):
                print

    def recommendCompletionsFor(self, word):
        parts = word.split(".")
        if len(parts) > 1:
            # has a . so it must be a module or class or something
            # using eval, which shouldn"t normally have side effects
            # unless there"s descriptors/metaclasses doing some nasty
            # get magic
            objname = ".".join(parts[:-1])
            try:
                obj = eval(objname, self.locals)
            except:  # noqa: E722, B001
                return None, 0
            wordlower = parts[-1].lower()
            if wordlower == "":
                # they just punched in a dot, so list all attributes
                # that don"t look private or special
                prefix = ".".join(parts[-2:])
                check = [
                    (prefix + _method)
                    for _method in dir(obj)
                    if _method[:1] != "_" and _method.lower().startswith(wordlower)
                ]
            else:
                # they started typing the method name
                check = filter(lambda s: s.lower().startswith(wordlower), dir(obj))
        else:
            # no dots, must be in the normal namespaces.. no eval necessary
            check = set(dir(__builtins__))
            check.update(keyword.kwlist)
            check.update(self.locals)
            wordlower = parts[-1].lower()
            check = filter(lambda s: s.lower().startswith(wordlower), check)
        check.sort()
        return check, 0


DEBUG_DELEGATE = 0
PASSTHROUGH = ("deleteBackward:", "complete:", "moveRight:", "moveLeft:")


class PyInterpreter(NSObject):
    """
    PyInterpreter is a delegate/controller for a NSTextView,
    turning it into a full featured interactive Python interpreter.
    """

    textView = objc.ivar("textView")

    def initWithTextView_(self, textView):
        self = super(PyInterpreter, self).init()
        self.textView = textView
        self.textView.setDelegate_(self)
        self.awakeFromNib()
        return self

    def interpreterLocals(self):
        return self._console.locals

    #
    #  NSApplicationDelegate methods
    #

    def applicationDidFinishLaunching_(self, aNotification):
        self.textView.setFont_(self.font())
        self.textView.setContinuousSpellCheckingEnabled_(False)
        self.textView.setRichText_(False)
        self._executeWithRedirectedIO(self._interp)

    #
    #  NIB loading protocol
    #

    def awakeFromNib(self):
        self = super(PyInterpreter, self).init()
        self._font = NSFont.userFixedPitchFontOfSize_(10)
        self._stderrColor = NSColor.redColor()
        self._stdoutColor = NSColor.blueColor()
        self._codeColor = NSColor.blackColor()
        self._historyLength = 50
        self._history = [""]
        self._historyView = 0
        self._characterIndexForInput = 0
        self._stdin = PseudoUTF8Input(self._nestedRunLoopReaderUntilEOLchars_)
        # self._stdin = PseudoUTF8Input(self.readStdin)
        self._stderr = PseudoUTF8Output(self.writeStderr_)
        self._stdout = PseudoUTF8Output(self.writeStdout_)
        self._isInteracting = False
        self._console = AsyncInteractiveConsole()
        self._interp = self._console.asyncinteract(write=self.writeCode_).next
        self._autoscroll = True
        self.applicationDidFinishLaunching_(None)

    #
    #  Modal input dialog support
    #

    def _nestedRunLoopReaderUntilEOLchars_(self, eolchars):
        """
        This makes the baby jesus cry.

        I want co-routines.
        """
        app = NSApplication.sharedApplication()
        window = self.textView.window()
        self.setCharacterIndexForInput_(self.lengthOfTextView())
        # change the color.. eh
        self.textView.setTypingAttributes_(
            {
                NSFontAttributeName: self.font(),
                NSForegroundColorAttributeName: self.codeColor(),
            }
        )
        while True:
            event = app.nextEventMatchingMask_untilDate_inMode_dequeue_(
                NSAnyEventMask, NSDate.distantFuture(), NSDefaultRunLoopMode, True
            )
            if (event.type() == NSKeyDown) and (event.window() == window):
                eol = event.characters()
                if eol in eolchars:
                    break
            app.sendEvent_(event)
        cl = self.currentLine()
        if eol == "\r":
            self.writeCode_("\n")
        return cl + eol

    #
    #  Interpreter functions
    #

    @objc.python_method
    def _executeWithRedirectedIO(self, fn, *args, **kwargs):
        old = sys.stdin, sys.stdout, sys.stderr
        if self._stdin is not None:
            sys.stdin = self._stdin
        sys.stdout, sys.stderr = self._stdout, self._stderr
        try:
            rval = fn(*args, **kwargs)
        finally:
            sys.stdin, sys.stdout, sys.stderr = old
            self.setCharacterIndexForInput_(self.lengthOfTextView())
        return rval

    def executeLine_(self, line):
        self.addHistoryLine_(line)
        self._executeWithRedirectedIO(self._executeLine_, line)
        self._history = filter(None, self._history)
        self._history.append("")
        self._historyView = len(self._history) - 1

    def _executeLine_(self, line):
        self._interp()(line)
        self._more = self._interp()

    def executeInteractiveLine_(self, line):
        self.setIsInteracting_(True)
        try:
            self.executeLine_(line)
        finally:
            self.setIsInteracting_(False)

    def replaceLineWithCode_(self, s):
        idx = self.characterIndexForInput()
        ts = self.textView.textStorage()
        ts.replaceCharactersInRange_withAttributedString_(
            (idx, len(ts.mutableString()) - idx), self.codeString_(s)
        )

    #
    #  History functions
    #

    def historyLength(self):
        return self._historyLength

    def setHistoryLength_(self, length):
        self._historyLength = length

    def addHistoryLine_(self, line):
        line = line.rstrip("\n")
        if self._history[-1] == line:
            return False
        if not line:
            return False
        self._history.append(line)
        if len(self._history) > self.historyLength():
            self._history.pop(0)
        return True

    def historyDown_(self, sender):
        if self._historyView == (len(self._history) - 1):
            return
        self._history[self._historyView] = self.currentLine()
        self._historyView += 1
        self.replaceLineWithCode_(self._history[self._historyView])
        self.moveToEndOfLine_(self)

    def historyUp_(self, sender):
        if self._historyView == 0:
            return
        self._history[self._historyView] = self.currentLine()
        self._historyView -= 1
        self.replaceLineWithCode_(self._history[self._historyView])
        self.moveToEndOfLine_(self)

    #
    #  Convenience methods to create/write decorated text
    #

    def _formatString_forOutput_(self, s, name):
        return NSAttributedString.alloc().initWithString_attributes_(
            s,
            {
                NSFontAttributeName: self.font(),
                NSForegroundColorAttributeName: getattr(self, name + "Color")(),
            },
        )

    def _writeString_forOutput_(self, s, name):
        self.textView.textStorage().appendAttributedString_(
            getattr(self, name + "String_")(s)
        )

        window = self.textView.window()
        app = NSApplication.sharedApplication()
        st = time.time()
        now = time.time

        if self._autoscroll:
            self.textView.scrollRangeToVisible_((self.lengthOfTextView(), 0))

        while app.isRunning() and now() - st < 0.01:
            event = app.nextEventMatchingMask_untilDate_inMode_dequeue_(
                NSAnyEventMask,
                NSDate.dateWithTimeIntervalSinceNow_(0.01),
                NSDefaultRunLoopMode,
                True,
            )

            if event is None:
                continue

            if (event.type() == NSKeyDown) and (event.window() == window):
                chars = event.charactersIgnoringModifiers()
                if chars == "c" and (event.modifierFlags() & NSControlKeyMask):
                    raise KeyboardInterrupt

            app.sendEvent_(event)

    def codeString_(self, s):
        return self._formatString_forOutput_(s, "code")

    def stderrString_(self, s):
        return self._formatString_forOutput_(s, "stderr")

    def stdoutString_(self, s):
        return self._formatString_forOutput_(s, "stdout")

    def writeCode_(self, s):
        return self._writeString_forOutput_(s, "code")

    def writeStderr_(self, s):
        return self._writeString_forOutput_(s, "stderr")

    def writeStdout_(self, s):
        return self._writeString_forOutput_(s, "stdout")

    #
    #  Accessors
    #

    def more(self):
        return self._more

    def font(self):
        return self._font

    def setFont_(self, font):
        self._font = font

    def stderrColor(self):
        return self._stderrColor

    def setStderrColor_(self, color):
        self._stderrColor = color

    def stdoutColor(self):
        return self._stdoutColor

    def setStdoutColor_(self, color):
        self._stdoutColor = color

    def codeColor(self):
        return self._codeColor

    def setCodeColor_(self, color):
        self._codeColor = color

    def isInteracting(self):
        return self._isInteracting

    def setIsInteracting_(self, v):
        self._isInteracting = v

    def isAutoScroll(self):
        return self._autoScroll

    def setAutoScroll_(self, v):
        self._autoScroll = v

    #
    #  Convenience methods for manipulating the NSTextView
    #

    def currentLine(self):
        return self.textView.textStorage().mutableString()[
            self.characterIndexForInput() :
        ]

    def moveAndScrollToIndex_(self, idx):
        self.textView.scrollRangeToVisible_((idx, 0))
        self.textView.setSelectedRange_((idx, 0))

    def characterIndexForInput(self):
        return self._characterIndexForInput

    def lengthOfTextView(self):
        return len(self.textView.textStorage().mutableString())

    def setCharacterIndexForInput_(self, idx):
        self._characterIndexForInput = idx
        self.moveAndScrollToIndex_(idx)

    #
    #  NSTextViewDelegate methods
    #

    def textView_completions_forPartialWordRange_indexOfSelectedItem_(
        self, aTextView, completions, begin_length, index
    ):
        begin, length = begin_length
        txt = self.textView.textStorage().mutableString()
        end = begin + length
        while (begin > 0) and (txt[begin].isalnum() or txt[begin] in "._"):
            begin -= 1
        while not txt[begin].isalnum():
            begin += 1
        return self._console.recommendCompletionsFor(txt[begin:end])

    def textView_shouldChangeTextInRange_replacementString_(
        self, aTextView, aRange, newString
    ):
        begin, length = aRange
        lastLocation = self.characterIndexForInput()
        if begin < lastLocation:
            # no editing anywhere but the interactive line
            return NO
        newString = newString.replace("\r", "\n")
        if "\n" in newString:
            if begin != lastLocation:
                # no pasting multiline unless you're at the end
                # of the interactive line
                return NO
            # multiline paste support
            # self.clearLine()
            newString = self.currentLine() + newString
            for s in newString.strip().split("\n"):
                self.writeCode_(s + "\n")
                self.executeLine_(s)
            return NO
        return YES

    def textView_willChangeSelectionFromCharacterRange_toCharacterRange_(
        self, aTextView, fromRange, toRange
    ):
        return toRange
        begin, length = toRange
        if length == 0 and begin < self.characterIndexForInput():
            # no cursor movement off the interactive line
            return fromRange
        return toRange

    def textView_doCommandBySelector_(self, aTextView, aSelector):
        # deleteForward: is ctrl-d
        if self.isInteracting():
            if aSelector == "insertNewline:":
                self.writeCode_("\n")
            return NO
        responder = getattr(self, aSelector.replace(":", "_"), None)
        if responder is not None:
            responder(aTextView)
            return YES
        else:
            if DEBUG_DELEGATE and aSelector not in PASSTHROUGH:
                print(aSelector)
            return NO

    #
    #  doCommandBySelector "posers" on the textView
    #

    def insertTabIgnoringFieldEditor_(self, sender):
        # this isn"t terribly necessary, b/c F5 and opt-esc do completion
        # but why not
        sender.complete_(self)

    def moveToBeginningOfLine_(self, sender):
        self.moveAndScrollToIndex_(self.characterIndexForInput())

    def moveToEndOfLine_(self, sender):
        self.moveAndScrollToIndex_(self.lengthOfTextView())

    def moveToBeginningOfLineAndModifySelection_(self, sender):
        begin, length = self.textView.selectedRange()
        pos = self.characterIndexForInput()
        if begin + length > pos:
            self.textView.setSelectedRange_((pos, begin + length - pos))
        else:
            self.moveToBeginningOfLine_(sender)

    def moveToEndOfLineAndModifySelection_(self, sender):
        begin, length = self.textView.selectedRange()
        pos = max(self.characterIndexForInput(), begin)
        self.textView.setSelectedRange_((pos, self.lengthOfTextView()))

    def insertNewline_(self, sender):
        line = self.currentLine()
        self.writeCode_("\n")
        self.executeInteractiveLine_(line)

    moveToBeginningOfParagraph_ = moveToBeginningOfLine_
    moveToEndOfParagraph_ = moveToEndOfLine_
    insertNewlineIgnoringFieldEditor_ = insertNewline_
    moveDown_ = historyDown_
    moveUp_ = historyUp_


class WebKitInterpreter(NSView):

    arguments = objc.ivar("arguments")
    pyInterpreter = objc.ivar("pyInterpreter")
    scrollView = objc.ivar("scrollView")
    textView = objc.ivar("textView")

    def container(self):
        return self.arguments.get("WebPluginContainer")

    def pluginViewWithArguments_(cls, arguments):
        self = super(WebKitInterpreter, cls).alloc().initWithFrame_(NSZeroRect)
        NSLog("pluginViewWithArguments:")
        NSLog(arguments)
        self.arguments = arguments
        return self

    pluginViewWithArguments_ = classmethod(pluginViewWithArguments_)

    def pluginStart(self):
        NSLog("pluginStart")
        try:
            self.doPluginStart()
        except:  # noqa: E722, B001
            traceback.print_exc()

    def doPluginStart(self):
        dct = self.arguments["WebPluginAttributes"]
        w, h = [float(dct.get(k, 0)) for k in ("width", "height")]

        self.setFrame_(((0.0, 0.0), (w, h)))
        scrollView = NSScrollView.alloc().initWithFrame_(self.frame())
        scrollView.setHasVerticalScroller_(True)
        scrollView.setHasHorizontalScroller_(False)
        scrollView.setAutoresizingMask_(NSViewWidthSizable | NSViewHeightSizable)
        contentSize = scrollView.contentSize()
        textView = NSTextView.alloc().initWithFrame_(((0, 0), scrollView.contentSize()))
        textView.setMinSize_((0, contentSize.height))
        textView.setMaxSize_((FLT_MAX, FLT_MAX))
        textView.setVerticallyResizable_(True)
        textView.setHorizontallyResizable_(False)
        textView.setAutoresizingMask_(NSViewWidthSizable)

        textView.textContainer().setContainerSize_((contentSize.width, FLT_MAX))
        textView.textContainer().setWidthTracksTextView_(True)

        scrollView.setDocumentView_(textView)
        self.addSubview_(scrollView)

        self.pyInterpreter = PyInterpreter.alloc().initWithTextView_(textView)

        self.pyInterpreter.interpreterLocals()["container"] = self.container()

    def objectForWebScript(self):
        return self


NSLog("loaded WebKitInterpreter")

objc.removeAutoreleasePool()



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKSnapshotConfiguration(TestCase):
    @min_os_level("10.15")
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.WKSnapshotConfiguration.afterScreenUpdates)
        self.assertArgIsBOOL(WebKit.WKSnapshotConfiguration.setAfterScreenUpdates_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKOpenPanelParameters(TestCase):
    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(WebKit.WKOpenPanelParameters.allowsMultipleSelection)

    @min_os_level("10.13.4")
    def testMethods10_13_4(self):
        self.assertResultIsBOOL(WebKit.WKOpenPanelParameters.allowsDirectories)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMCSSPrimitiveValue(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_CSS_UNKNOWN, 0)
        self.assertEqual(WebKit.DOM_CSS_NUMBER, 1)
        self.assertEqual(WebKit.DOM_CSS_PERCENTAGE, 2)
        self.assertEqual(WebKit.DOM_CSS_EMS, 3)
        self.assertEqual(WebKit.DOM_CSS_EXS, 4)
        self.assertEqual(WebKit.DOM_CSS_PX, 5)
        self.assertEqual(WebKit.DOM_CSS_CM, 6)
        self.assertEqual(WebKit.DOM_CSS_MM, 7)
        self.assertEqual(WebKit.DOM_CSS_IN, 8)
        self.assertEqual(WebKit.DOM_CSS_PT, 9)
        self.assertEqual(WebKit.DOM_CSS_PC, 10)
        self.assertEqual(WebKit.DOM_CSS_DEG, 11)
        self.assertEqual(WebKit.DOM_CSS_RAD, 12)
        self.assertEqual(WebKit.DOM_CSS_GRAD, 13)
        self.assertEqual(WebKit.DOM_CSS_MS, 14)
        self.assertEqual(WebKit.DOM_CSS_S, 15)
        self.assertEqual(WebKit.DOM_CSS_HZ, 16)
        self.assertEqual(WebKit.DOM_CSS_KHZ, 17)
        self.assertEqual(WebKit.DOM_CSS_DIMENSION, 18)
        self.assertEqual(WebKit.DOM_CSS_STRING, 19)
        self.assertEqual(WebKit.DOM_CSS_URI, 20)
        self.assertEqual(WebKit.DOM_CSS_IDENT, 21)
        self.assertEqual(WebKit.DOM_CSS_ATTR, 22)
        self.assertEqual(WebKit.DOM_CSS_COUNTER, 23)
        self.assertEqual(WebKit.DOM_CSS_RECT, 24)
        self.assertEqual(WebKit.DOM_CSS_RGBCOLOR, 25)
        self.assertEqual(WebKit.DOM_CSS_VW, 26)
        self.assertEqual(WebKit.DOM_CSS_VH, 27)
        self.assertEqual(WebKit.DOM_CSS_VMIN, 28)
        self.assertEqual(WebKit.DOM_CSS_VMAX, 29)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKFindConfiguration(TestCase):
    @min_os_level("10.16")
    def testMethods10_16(self):
        self.assertResultIsBOOL(WebKit.WKFindConfiguration.backwards)
        self.assertArgIsBOOL(WebKit.WKFindConfiguration.setBackwards_, 0)

        self.assertResultIsBOOL(WebKit.WKFindConfiguration.caseSensitive)
        self.assertArgIsBOOL(WebKit.WKFindConfiguration.setCaseSensitive_, 0)

        self.assertResultIsBOOL(WebKit.WKFindConfiguration.wraps)
        self.assertArgIsBOOL(WebKit.WKFindConfiguration.setWraps_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKWebsiteDataStore(TestCase):
    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(WebKit.WKWebsiteDataStore.isPersistent)

        self.assertArgIsBlock(
            WebKit.WKWebsiteDataStore.fetchDataRecordsOfTypes_completionHandler_,
            1,
            b"v@",
        )
        self.assertArgIsBlock(
            WebKit.WKWebsiteDataStore.removeDataOfTypes_forDataRecords_completionHandler_,
            2,
            b"v",
        )
        self.assertArgIsBlock(
            WebKit.WKWebsiteDataStore.removeDataOfTypes_modifiedSince_completionHandler_,
            2,
            b"v",
        )



# This file is generated by objective.metadata
#
# Last update: Sat Jun 27 17:37:07 2020
#
# flake8: noqa

import objc, sys

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
constants = """$JSPropertyDescriptorConfigurableKey$JSPropertyDescriptorEnumerableKey$JSPropertyDescriptorGetKey$JSPropertyDescriptorSetKey$JSPropertyDescriptorValueKey$JSPropertyDescriptorWritableKey$"""
enums = """$JSC_OBJC_API_ENABLED@1$WEBKIT_VERSION_1_0@256$WEBKIT_VERSION_1_1@272$WEBKIT_VERSION_1_2@288$WEBKIT_VERSION_1_3@304$WEBKIT_VERSION_2_0@512$WEBKIT_VERSION_3_0@768$WEBKIT_VERSION_3_1@784$WEBKIT_VERSION_4_0@1024$WEBKIT_VERSION_LATEST@39321$kJSClassAttributeNoAutomaticPrototype@2$kJSClassAttributeNone@0$kJSPropertyAttributeDontDelete@8$kJSPropertyAttributeDontEnum@4$kJSPropertyAttributeNone@0$kJSPropertyAttributeReadOnly@2$kJSTypeBoolean@2$kJSTypeNull@1$kJSTypeNumber@3$kJSTypeObject@5$kJSTypeString@4$kJSTypeSymbol@6$kJSTypeUndefined@0$kJSTypedArrayTypeArrayBuffer@9$kJSTypedArrayTypeFloat32Array@7$kJSTypedArrayTypeFloat64Array@8$kJSTypedArrayTypeInt16Array@1$kJSTypedArrayTypeInt32Array@2$kJSTypedArrayTypeInt8Array@0$kJSTypedArrayTypeNone@10$kJSTypedArrayTypeUint16Array@5$kJSTypedArrayTypeUint32Array@6$kJSTypedArrayTypeUint8Array@3$kJSTypedArrayTypeUint8ClampedArray@4$"""
misc.update({})
functions = {
    "JSObjectSetPropertyForKey": (
        b"v^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}^{OpaqueJSValue=}I^^{OpaqueJSValue=}",
        "",
        {"arguments": {5: {"type_modifier": "o"}}},
    ),
    "JSClassRetain": (b"^{OpaqueJSClass=}^{OpaqueJSClass=}",),
    "JSValueCreateJSONString": (
        b"^{OpaqueJSString=}^{OpaqueJSContext=}^{OpaqueJSValue=}I^^{OpaqueJSValue=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSObjectDeletePropertyForKey": (
        b"B^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "JSValueGetTypedArrayType": (
        b"I^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "JSStringCreateWithCFString": (
        b"^{OpaqueJSString=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSValueToBoolean": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSStringCopyCFString": (
        b"^{__CFString=}^{__CFAllocator=}^{OpaqueJSString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSObjectMakeTypedArray": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}IL^^{OpaqueJSValue=}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "JSValueMakeString": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSString=}",),
    "JSObjectGetArrayBufferBytesPtr": (
        b"^v^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {
            "retval": {"c_array_of_variable_length": True},
            "arguments": {2: {"type_modifier": "o"}},
        },
    ),
    "JSObjectGetTypedArrayByteOffset": (
        b"Q^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "JSValueMakeNumber": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}d",),
    "JSObjectMakeError": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}Q^^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {
            "arguments": {
                2: {"c_array_length_in_arg": 1, "type_modifier": "n"},
                3: {"type_modifier": "o"},
            }
        },
    ),
    "JSValueProtect": (b"v^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSValueIsDate": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSObjectGetTypedArrayBytesPtr": (
        b"^v^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {
            "retval": {"c_array_of_variable_length": True},
            "arguments": {2: {"type_modifier": "o"}},
        },
    ),
    "JSValueToStringCopy": (
        b"^{OpaqueJSString=}^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSValueIsNumber": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSObjectHasProperty": (
        b"B^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSString=}",
    ),
    "JSValueIsEqual": (
        b"B^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
    ),
    "JSValueIsArray": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSValueToNumber": (b"d^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",),
    "JSObjectMakeDeferredPromise": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^^{OpaqueJSValue=}^^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {1: {"type_modifier": "o"}, 2: {"type_modifier": "o"}}},
    ),
    "JSObjectGetTypedArrayByteLength": (
        b"Q^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "JSValueUnprotect": (b"v^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSObjectSetPropertyAtIndex": (
        b"v^{OpaqueJSContext=}^{OpaqueJSValue=}I^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {4: {"type_modifier": "o"}}},
    ),
    "JSObjectGetTypedArrayLength": (
        b"Q^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "JSObjectCallAsFunction": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}Q^^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {
            "arguments": {
                4: {"c_array_length_in_arg": 3, "type_modifier": "n"},
                5: {"type_modifier": "o"},
            }
        },
    ),
    "JSPropertyNameArrayGetCount": (b"Q^{OpaqueJSPropertyNameArray=}",),
    "JSStringGetLength": (b"Q^{OpaqueJSString=}",),
    "JSValueToObject": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
    ),
    "JSValueIsString": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSObjectCopyPropertyNames": (
        b"^{OpaqueJSPropertyNameArray=}^{OpaqueJSContext=}^{OpaqueJSValue=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSGlobalContextCopyName": (
        b"^{OpaqueJSString=}^{OpaqueJSContext=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSClassRelease": (b"v^{OpaqueJSClass=}",),
    "JSValueMakeUndefined": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}",),
    "JSObjectMakeArray": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}Q^^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {
            "arguments": {
                2: {"c_array_length_in_arg": 1, "type_modifier": "n"},
                3: {"type_modifier": "o"},
            }
        },
    ),
    "JSGlobalContextSetName": (b"v^{OpaqueJSContext=}^{OpaqueJSString=}",),
    "JSStringGetMaximumUTF8CStringSize": (b"Q^{OpaqueJSString=}",),
    "JSStringCreateWithCharacters": (
        b"^{OpaqueJSString=}^TQ",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {0: {"c_array_length_in_arg": 1, "type_modifier": "n"}},
        },
    ),
    "JSValueIsSymbol": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSObjectMakeTypedArrayWithArrayBufferAndOffset": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}I^{OpaqueJSValue=}QQ^^{OpaqueJSValue=}",
        "",
        {"arguments": {5: {"type_modifier": "o"}}},
    ),
    "JSObjectCallAsConstructor": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSValue=}Q^^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {
            "arguments": {
                3: {"c_array_length_in_arg": 2, "type_modifier": "n"},
                4: {"type_modifier": "o"},
            }
        },
    ),
    "JSObjectIsFunction": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSValueIsInstanceOfConstructor": (
        b"B^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
    ),
    "JSCheckScriptSyntax": (
        b"B^{OpaqueJSContext=}^{OpaqueJSString=}^{OpaqueJSString=}i^^{OpaqueJSValue=}",
        "",
        {"arguments": {4: {"type_modifier": "o"}}},
    ),
    "JSObjectIsConstructor": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSPropertyNameArrayRelease": (b"v^{OpaqueJSPropertyNameArray=}",),
    "JSValueIsBoolean": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSContextGroupRetain": (b"^{OpaqueJSContextGroup=}^{OpaqueJSContextGroup=}",),
    "JSGlobalContextRetain": (b"^{OpaqueJSContext=}^{OpaqueJSContext=}",),
    "JSStringRelease": (b"v^{OpaqueJSString=}",),
    "JSObjectMakeFunctionWithCallback": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSString=}^?",
        "",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"^{OpaqueJSValue=}"},
                        "arguments": {
                            0: {"type": b"^{OpaqueJSContext=}"},
                            1: {"type": b"^{OpaqueJSValue=}"},
                            2: {"type": b"^{OpaqueJSValue=}"},
                            3: {"type": b"L"},
                            4: {
                                "type": b"^^{OpaqueJSValue=}",
                                "type_modifier": "n",
                                "c_array_length_in_arg": 3,
                            },
                            5: {"type": b"^^{OpaqueJSValue=}", "type_modifier": "o"},
                        },
                    }
                }
            }
        },
    ),
    "JSValueIsObject": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSStringGetUTF8CString": (
        b"Q^{OpaqueJSString=}^tQ",
        "",
        {
            "arguments": {
                1: {
                    "c_array_length_in_result": True,
                    "type_modifier": "o",
                    "c_array_length_in_arg": 2,
                }
            }
        },
    ),
    "JSGlobalContextCreate": (
        b"^{OpaqueJSContext=}^{OpaqueJSClass=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSObjectSetProperty": (
        b"v^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSString=}^{OpaqueJSValue=}I^^{OpaqueJSValue=}",
        "",
        {"arguments": {5: {"type_modifier": "o"}}},
    ),
    "JSValueMakeBoolean": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}B",),
    "JSGlobalContextRelease": (b"v^{OpaqueJSContext=}",),
    "JSObjectMakeRegExp": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}Q^^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {
            "arguments": {
                2: {"c_array_length_in_arg": 1, "type_modifier": "n"},
                3: {"type_modifier": "o"},
            }
        },
    ),
    "JSStringIsEqualToUTF8CString": (
        b"B^{OpaqueJSString=}^t",
        "",
        {"arguments": {1: {"c_array_delimited_by_null": True, "type_modifier": "n"}}},
    ),
    "JSObjectGetProperty": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSString=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "JSObjectMakeArrayBufferWithBytesNoCopy": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^vQ^?^v^^{OpaqueJSValue=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                1: {"c_array_length_in_arg": 2, "type_modifier": "n"},
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"^v"}},
                    },
                    "callable_retained": True,
                },
                5: {"type_modifier": "o"},
            },
        },
    ),
    "JSValueIsNull": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSPropertyNameAccumulatorAddName": (
        b"v^{OpaqueJSPropertyNameAccumulator=}^{OpaqueJSString=}",
    ),
    "JSStringRetain": (b"^{OpaqueJSString=}^{OpaqueJSString=}",),
    "JSObjectHasPropertyForKey": (
        b"B^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "JSObjectMakeConstructor": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSClass=}^?",
        "",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"^{OpaqueJSValue=}"},
                        "arguments": {
                            0: {"type": b"^{OpaqueJSContext=}"},
                            1: {"type": b"^{OpaqueJSValue=}"},
                            2: {"type": b"L"},
                            3: {
                                "type": b"^^{OpaqueJSValue=}",
                                "type_modifier": "n",
                                "c_array_length_in_arg": 2,
                            },
                            4: {"type": b"^^{OpaqueJSValue=}", "type_modifier": "o"},
                        },
                    }
                }
            }
        },
    ),
    "JSObjectSetPrivate": (b"B^{OpaqueJSValue=}^v",),
    "JSObjectMakeTypedArrayWithBytesNoCopy": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}I^vL^?^v^^{OpaqueJSValue=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                2: {"c_array_length_in_arg": 3, "type_modifier": "n"},
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"^v"}},
                    }
                },
                6: {"type_modifier": "o"},
            },
        },
    ),
    "JSValueMakeFromJSONString": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSString=}",
    ),
    "JSEvaluateScript": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSString=}^{OpaqueJSValue=}^{OpaqueJSString=}i^^{OpaqueJSValue=}",
        "",
        {"arguments": {5: {"type_modifier": "o"}}},
    ),
    "JSContextGetGlobalContext": (b"^{OpaqueJSContext=}^{OpaqueJSContext=}",),
    "JSStringCreateWithUTF8CString": (
        b"^{OpaqueJSString=}^t",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {0: {"c_array_delimited_by_null": True, "type_modifier": "n"}},
        },
    ),
    "JSObjectGetTypedArrayBuffer": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "JSObjectGetArrayBufferByteLength": (
        b"Q^{OpaqueJSContext=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "JSContextGetGlobalObject": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}",),
    "JSPropertyNameArrayRetain": (
        b"^{OpaqueJSPropertyNameArray=}^{OpaqueJSPropertyNameArray=}",
    ),
    "JSObjectMake": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSClass=}^v",),
    "JSObjectGetPrivate": (b"^v^{OpaqueJSValue=}",),
    "JSValueIsUndefined": (b"B^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSValueIsObjectOfClass": (
        b"B^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSClass=}",
    ),
    "JSObjectGetPrototype": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSValue=}",),
    "JSPropertyNameArrayGetNameAtIndex": (
        b"^{OpaqueJSString=}^{OpaqueJSPropertyNameArray=}Q",
    ),
    "JSValueMakeNull": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}",),
    "JSContextGetGroup": (b"^{OpaqueJSContextGroup=}^{OpaqueJSContext=}",),
    "JSContextGroupCreate": (
        b"^{OpaqueJSContextGroup=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSObjectDeleteProperty": (
        b"B^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSString=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "JSObjectGetPropertyForKey": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "JSObjectMakeFunction": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSString=}I^^{OpaqueJSString=}^{OpaqueJSString=}^{OpaqueJSString=}i^^{OpaqueJSValue=}",
        "",
        {
            "arguments": {
                3: {"c_array_length_in_arg": 2, "type_modifier": "n"},
                7: {"type_modifier": "o"},
            }
        },
    ),
    "JSObjectGetPropertyAtIndex": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSValue=}I^^{OpaqueJSValue=}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "JSStringIsEqual": (b"B^{OpaqueJSString=}^{OpaqueJSString=}",),
    "JSObjectSetPrototype": (
        b"v^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}",
    ),
    "JSContextGroupRelease": (b"v^{OpaqueJSContextGroup=}",),
    "JSValueIsStrictEqual": (
        b"B^{OpaqueJSContext=}^{OpaqueJSValue=}^{OpaqueJSValue=}",
    ),
    "JSGlobalContextCreateInGroup": (
        b"^{OpaqueJSContext=}^{OpaqueJSContextGroup=}^{OpaqueJSClass=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "JSObjectMakeDate": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}Q^^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {
            "arguments": {
                2: {"c_array_length_in_arg": 1, "type_modifier": "n"},
                3: {"type_modifier": "o"},
            }
        },
    ),
    "JSObjectMakeTypedArrayWithArrayBuffer": (
        b"^{OpaqueJSValue=}^{OpaqueJSContext=}I^{OpaqueJSValue=}^^{OpaqueJSValue=}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "JSValueMakeSymbol": (b"^{OpaqueJSValue=}^{OpaqueJSContext=}^{OpaqueJSString=}",),
    "JSGarbageCollect": (b"v^{OpaqueJSContext=}",),
    "JSValueGetType": (b"I^{OpaqueJSContext=}^{OpaqueJSValue=}",),
}
aliases = {"JSObjectRef": "JSValueRef", "JSGlobalContextRef": "JSContextRef"}
misc.update(
    {
        "JSValueRef": objc.createOpaquePointerType("JSValueRef", b"^{OpaqueJSValue=}"),
        "JSStringRef": objc.createOpaquePointerType(
            "JSStringRef", b"^{OpaqueJSString=}"
        ),
        "JSContextRef": objc.createOpaquePointerType(
            "JSContextRef", b"^{OpaqueJSContext=}"
        ),
        "JSPropertyNameArrayRef": objc.createOpaquePointerType(
            "JSPropertyNameArrayRef", b"^{OpaqueJSPropertyNameArray=}"
        ),
        "JSClassRef": objc.createOpaquePointerType("JSClassRef", b"^{OpaqueJSClass=}"),
        "JSContextGroupRef": objc.createOpaquePointerType(
            "JSContextGroupRef", b"^{OpaqueJSContextGroup=}"
        ),
        "JSPropertyNameAccumulatorRef": objc.createOpaquePointerType(
            "JSPropertyNameAccumulatorRef", b"^{OpaqueJSPropertyNameAccumulator=}"
        ),
    }
)
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"JSContext",
        b"exceptionHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                    },
                }
            }
        },
    )
    r(
        b"JSContext",
        b"setExceptionHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(b"JSValue", b"deleteProperty:", {"retval": {"type": "Z"}})
    r(b"JSValue", b"hasProperty:", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isArray", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isBoolean", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isDate", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isEqualToObject:", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isEqualWithTypeCoercionToObject:", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isInstanceOf:", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isNull", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isNumber", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isObject", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isString", {"retval": {"type": "Z"}})
    r(b"JSValue", b"isSymbol", {"retval": {"type": b"Z"}})
    r(b"JSValue", b"isUndefined", {"retval": {"type": "Z"}})
    r(b"JSValue", b"toBool", {"retval": {"type": "Z"}})
    r(b"JSValue", b"valueWithBool:inContext:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"JSValue",
        b"valueWithNewPromiseInContext:fromExecutor:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLLinkElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLLinkElement.disabled)
        self.assertArgIsBOOL(WebKit.DOMHTMLLinkElement.setDisabled_, 0)



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

plist = {
    "NSMainNibFile": "PyDocBrowser",
    "NSAppleScriptEnabled": True,
    "CFBundleURLTypes": [
        {"CFBundleURLName": "Python Documention URL", "CFBundleURLSchemes": ["pydoc"]}
    ],
}

setup(
    app=["PyDocBrowser.py"],
    data_files=["PyDocBrowser.nib"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa", "pyobjc-framework-WebKit"],
)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLImageElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLImageElement.isMap)
        self.assertArgIsBOOL(WebKit.DOMHTMLImageElement.setIsMap_, 0)

        self.assertResultIsBOOL(WebKit.DOMHTMLImageElement.complete)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLObjectElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLObjectElement.declare)
        self.assertArgIsBOOL(WebKit.DOMHTMLObjectElement.setDeclare_, 0)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMNodeIterator(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMNodeIterator.expandEntityReferences)

        self.assertResultIsBOOL(WebKit.DOMNodeIterator.pointerBeforeReferenceNode)



import JavaScriptCore  # noqa: F401
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestJSExport(TestCase):
    @min_os_level("10.9")
    def test_protocols(self):
        import sysconfig

        target = sysconfig.get_config_var("MACOSX_DEPLOYMENT_TARGET")
        if target in ("10.6", "10.7", "10.9"):
            # The protocol is not available for older binaries...
            return

        self.assertIsInstance(objc.protocolNamed("JSExport"), objc.formal_protocol)

    # XXX: JSExportAs support



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit  # noqa: F401
import objc


class TestWKUIDelegateHelper(WebKit.NSObject):
    def webView_createWebViewWithConfiguration_forNavigationAction_windowFeatures_(
        self, w, c, a, f
    ):
        return None

    def webView_runJavaScriptAlertPanelWithMessage_initiatedByFrame_completionHandler_(
        self, w, m, f, h
    ):
        pass

    def webView_runJavaScriptConfirmPanelWithMessage_initiatedByFrame_completionHandler_(
        self, w, m, f, h
    ):
        pass

    def webView_runJavaScriptTextInputPanelWithPrompt_defaultText_initiatedByFrame_completionHandler_(  # noqa: B950
        self, w, p, d, f, h
    ):
        pass

    def webView_runOpenPanelWithParameters_initiatedByFrame_completionHandler_(
        self, w, p, f, c
    ):
        pass


class TestWKUIDelegate(TestCase):
    @min_os_level("10.10")
    def testProtocols10_10(self):
        p = objc.protocolNamed("WKUIDelegate")
        self.assertIsInstance(p, objc.formal_protocol)

        self.assertArgIsBlock(
            TestWKUIDelegateHelper.webView_runJavaScriptAlertPanelWithMessage_initiatedByFrame_completionHandler_,  # noqa: B950
            3,
            b"v",
        )
        self.assertArgIsBlock(
            TestWKUIDelegateHelper.webView_runJavaScriptAlertPanelWithMessage_initiatedByFrame_completionHandler_,  # noqa: B950
            3,
            b"v",
        )
        self.assertArgIsBlock(
            TestWKUIDelegateHelper.webView_runJavaScriptConfirmPanelWithMessage_initiatedByFrame_completionHandler_,  # noqa: B950
            3,
            b"v",
        )
        self.assertArgIsBlock(
            TestWKUIDelegateHelper.webView_runJavaScriptTextInputPanelWithPrompt_defaultText_initiatedByFrame_completionHandler_,  # noqa: B950
            4,
            b"v",
        )
        self.assertArgIsBlock(
            TestWKUIDelegateHelper.webView_runOpenPanelWithParameters_initiatedByFrame_completionHandler_,  # noqa: B950
            3,
            b"v@",
        )



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLDListElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLDListElement.compact)
        self.assertArgIsBOOL(WebKit.DOMHTMLDListElement.setCompact_, 0)



"""
Python mapping for the WebKit framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import Foundation
import objc
from WebKit import _metadata
from WebKit import _WebKit

objc.addConvenienceForBasicSequence("WebScriptObject", True)

sys.modules["WebKit"] = mod = objc.ObjCLazyModule(
    "WebKit",
    "com.apple.WebKit",
    objc.pathForFramework("/System/Library/Frameworks/WebKit.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "objc": objc,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (_WebKit, Foundation),
)


del sys.modules["WebKit._metadata"]



"""
Python mapping for the JavaScriptCore framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import CoreFoundation
import JavaScriptCore._util
import objc
from JavaScriptCore import _metadata

sys.modules["JavaScriptCore"] = mod = objc.ObjCLazyModule(
    "JavaScriptCore",
    "com.apple.JavaScriptCore",
    objc.pathForFramework("/System/Library/Frameworks/JavaScriptCore.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "objc": objc,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (CoreFoundation,),
)


del sys.modules["JavaScriptCore._metadata"]


mod.autoreleasing = JavaScriptCore._util.autoreleasing



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import WebKit
import objc


class TestWebPolicyDelegate(TestCase):
    def testConstants(self):
        self.assertIsInstance(WebKit.WebActionNavigationTypeKey, str)
        self.assertIsInstance(WebKit.WebActionElementKey, str)
        self.assertIsInstance(WebKit.WebActionButtonKey, str)
        self.assertIsInstance(WebKit.WebActionModifierFlagsKey, str)
        self.assertIsInstance(WebKit.WebActionOriginalURLKey, str)

        self.assertEqual(WebKit.WebNavigationTypeLinkClicked, 0)
        self.assertEqual(WebKit.WebNavigationTypeFormSubmitted, 1)
        self.assertEqual(WebKit.WebNavigationTypeBackForward, 2)
        self.assertEqual(WebKit.WebNavigationTypeReload, 3)
        self.assertEqual(WebKit.WebNavigationTypeFormResubmitted, 4)
        self.assertEqual(WebKit.WebNavigationTypeOther, 5)

    def testProtocols(self):
        self.assertIsInstance(
            objc.protocolNamed("WebPolicyDecisionListener"), objc.formal_protocol
        )

    @min_sdk_level("10.11")
    def testProtocols10_11(self):
        self.assertIsInstance(
            objc.protocolNamed("WebPolicyDelegate"), objc.formal_protocol
        )



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMRange(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_START_TO_START, 0)
        self.assertEqual(WebKit.DOM_START_TO_END, 1)
        self.assertEqual(WebKit.DOM_END_TO_END, 2)
        self.assertEqual(WebKit.DOM_END_TO_START, 3)
        self.assertEqual(WebKit.DOM_NODE_BEFORE, 0)
        self.assertEqual(WebKit.DOM_NODE_AFTER, 1)
        self.assertEqual(WebKit.DOM_NODE_BEFORE_AND_AFTER, 2)
        self.assertEqual(WebKit.DOM_NODE_INSIDE, 3)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMRange.collapsed)
        self.assertArgIsBOOL(WebKit.DOMRange.collapse_, 0)

        self.assertResultIsBOOL(WebKit.DOMRange.intersectsNode_)
        self.assertResultIsBOOL(WebKit.DOMRange.isPointInRange_offset_)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMCSSRule(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_UNKNOWN_RULE, 0)
        self.assertEqual(WebKit.DOM_STYLE_RULE, 1)
        self.assertEqual(WebKit.DOM_CHARSET_RULE, 2)
        self.assertEqual(WebKit.DOM_IMPORT_RULE, 3)
        self.assertEqual(WebKit.DOM_MEDIA_RULE, 4)
        self.assertEqual(WebKit.DOM_FONT_FACE_RULE, 5)
        self.assertEqual(WebKit.DOM_PAGE_RULE, 6)
        self.assertEqual(WebKit.DOM_VARIABLES_RULE, 7)  # Removed in 10.10(?)
        self.assertEqual(WebKit.DOM_KEYFRAMES_RULE, 7)
        self.assertEqual(WebKit.DOM_WEBKIT_KEYFRAMES_RULE, 7)
        self.assertEqual(WebKit.DOM_WEBKIT_KEYFRAME_RULE, 8)
        self.assertEqual(WebKit.DOM_KEYFRAME_RULE, 8)
        self.assertEqual(WebKit.DOM_SUPPORTS_RULE, 12)
        self.assertEqual(WebKit.DOM_WEBKIT_REGION_RULE, 16)

        self.assertEqual(WebKit.DOM_NAMESPACE_RULE, 10)



import JavaScriptCore
import objc
from PyObjCTools.TestSupport import TestCase


class TestJSBase(TestCase):
    def test_types(self):
        self.assertIsOpaquePointer(JavaScriptCore.JSContextGroupRef)
        self.assertIsOpaquePointer(JavaScriptCore.JSContextRef)
        self.assertIsOpaquePointer(JavaScriptCore.JSStringRef)
        self.assertIsOpaquePointer(JavaScriptCore.JSPropertyNameArrayRef)
        self.assertIsOpaquePointer(JavaScriptCore.JSPropertyNameAccumulatorRef)
        self.assertIsOpaquePointer(JavaScriptCore.JSValueRef)
        self.assertIsOpaquePointer(JavaScriptCore.JSClassRef)

        self.assertIs(JavaScriptCore.JSObjectRef, JavaScriptCore.JSValueRef)
        self.assertIs(JavaScriptCore.JSGlobalContextRef, JavaScriptCore.JSContextRef)

    def test_functions(self):
        self.assertResultHasType(
            JavaScriptCore.JSEvaluateScript, JavaScriptCore.JSValueRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSEvaluateScript, 0, JavaScriptCore.JSContextRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSEvaluateScript, 1, JavaScriptCore.JSStringRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSEvaluateScript, 2, JavaScriptCore.JSValueRef.__typestr__
        )
        self.assertArgHasType(
            JavaScriptCore.JSEvaluateScript, 3, JavaScriptCore.JSStringRef.__typestr__
        )
        self.assertArgHasType(JavaScriptCore.JSEvaluateScript, 4, objc._C_INT)
        self.assertArgHasType(
            JavaScriptCore.JSEvaluateScript,
            5,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(JavaScriptCore.JSCheckScriptSyntax, objc._C_BOOL)
        self.assertArgIsOut(JavaScriptCore.JSCheckScriptSyntax, 4)

        self.assertArgHasType(
            JavaScriptCore.JSGarbageCollect, 0, JavaScriptCore.JSContextRef.__typestr__
        )



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMXPathException(TestCase):
    def testConstants(self):
        self.assertIsInstance(WebKit.DOMXPathException, str)

        self.assertEqual(WebKit.DOM_INVALID_EXPRESSION_ERR, 51)
        self.assertEqual(WebKit.DOM_TYPE_ERR, 52)



from PyObjCTools.TestSupport import TestCase
import objc
import WebKit


class TestWebScriptObjectHelper(WebKit.NSObject):
    @classmethod
    def webScriptNameForSelector_(self, sel):
        return 1

    @classmethod
    def isSelectorExcludedFromWebScript_(self, sel):
        return 1

    @classmethod
    def webScriptNameForKey_(self, name):
        return 1

    @classmethod
    def isKeyExcludedFromWebScript_(self, name):
        return 1


class TestWebScriptObject(TestCase):
    def testMethods(self):
        self.assertArgHasType(
            TestWebScriptObjectHelper.webScriptNameForSelector_, 0, objc._C_SEL
        )
        self.assertArgHasType(
            TestWebScriptObjectHelper.isSelectorExcludedFromWebScript_, 0, objc._C_SEL
        )
        self.assertArgHasType(
            TestWebScriptObjectHelper.webScriptNameForKey_,
            0,
            b"n^" + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgIsNullTerminated(
            TestWebScriptObjectHelper.webScriptNameForKey_, 0
        )
        self.assertArgHasType(
            TestWebScriptObjectHelper.isKeyExcludedFromWebScript_,
            0,
            b"n^" + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgIsNullTerminated(
            TestWebScriptObjectHelper.isKeyExcludedFromWebScript_, 0
        )

        self.assertResultIsBOOL(WebKit.WebScriptObject.throwException_)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLDirectoryElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLDirectoryElement.compact)
        self.assertArgIsBOOL(WebKit.DOMHTMLDirectoryElement.setCompact_, 0)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMException(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_INDEX_SIZE_ERR, 1)
        self.assertEqual(WebKit.DOM_DOMSTRING_SIZE_ERR, 2)
        self.assertEqual(WebKit.DOM_HIERARCHY_REQUEST_ERR, 3)
        self.assertEqual(WebKit.DOM_WRONG_DOCUMENT_ERR, 4)
        self.assertEqual(WebKit.DOM_INVALID_CHARACTER_ERR, 5)
        self.assertEqual(WebKit.DOM_NO_DATA_ALLOWED_ERR, 6)
        self.assertEqual(WebKit.DOM_NO_MODIFICATION_ALLOWED_ERR, 7)
        self.assertEqual(WebKit.DOM_NOT_FOUND_ERR, 8)
        self.assertEqual(WebKit.DOM_NOT_SUPPORTED_ERR, 9)
        self.assertEqual(WebKit.DOM_INUSE_ATTRIBUTE_ERR, 10)
        self.assertEqual(WebKit.DOM_INVALID_STATE_ERR, 11)
        self.assertEqual(WebKit.DOM_SYNTAX_ERR, 12)
        self.assertEqual(WebKit.DOM_INVALID_MODIFICATION_ERR, 13)
        self.assertEqual(WebKit.DOM_NAMESPACE_ERR, 14)
        self.assertEqual(WebKit.DOM_INVALID_ACCESS_ERR, 15)

        self.assertIsInstance(WebKit.DOMException, str)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWebPreferences(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.WebCacheModelDocumentViewer, 0)
        self.assertEqual(WebKit.WebCacheModelDocumentBrowser, 1)
        self.assertEqual(WebKit.WebCacheModelPrimaryWebBrowser, 2)

        self.assertIsInstance(WebKit.WebPreferencesChangedNotification, str)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.WebPreferences.userStyleSheetEnabled)
        self.assertArgIsBOOL(WebKit.WebPreferences.setUserStyleSheetEnabled_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.isJavaEnabled)
        self.assertArgIsBOOL(WebKit.WebPreferences.setJavaEnabled_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.isJavaScriptEnabled)
        self.assertArgIsBOOL(WebKit.WebPreferences.setJavaScriptEnabled_, 0)

        self.assertResultIsBOOL(
            WebKit.WebPreferences.javaScriptCanOpenWindowsAutomatically
        )
        self.assertArgIsBOOL(
            WebKit.WebPreferences.setJavaScriptCanOpenWindowsAutomatically_, 0
        )

        self.assertResultIsBOOL(WebKit.WebPreferences.arePlugInsEnabled)
        self.assertArgIsBOOL(WebKit.WebPreferences.setPlugInsEnabled_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.allowsAnimatedImages)
        self.assertArgIsBOOL(WebKit.WebPreferences.setAllowsAnimatedImages_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.allowsAnimatedImageLooping)
        self.assertArgIsBOOL(WebKit.WebPreferences.setAllowsAnimatedImageLooping_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.loadsImagesAutomatically)
        self.assertArgIsBOOL(WebKit.WebPreferences.setLoadsImagesAutomatically_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.autosaves)
        self.assertArgIsBOOL(WebKit.WebPreferences.setAutosaves_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.shouldPrintBackgrounds)
        self.assertArgIsBOOL(WebKit.WebPreferences.setShouldPrintBackgrounds_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.privateBrowsingEnabled)
        self.assertArgIsBOOL(WebKit.WebPreferences.setPrivateBrowsingEnabled_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.tabsToLinks)
        self.assertArgIsBOOL(WebKit.WebPreferences.setTabsToLinks_, 0)

        self.assertResultIsBOOL(WebKit.WebPreferences.usesPageCache)
        self.assertArgIsBOOL(WebKit.WebPreferences.setUsesPageCache_, 0)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(WebKit.WebPreferences.suppressesIncrementalRendering)
        self.assertArgIsBOOL(
            WebKit.WebPreferences.setSuppressesIncrementalRendering_, 0
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(WebKit.WebPreferences.allowsAirPlayForMediaPlayback)
        self.assertArgIsBOOL(WebKit.WebPreferences.setAllowsAirPlayForMediaPlayback_, 0)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestWebHistoryItem(TestCase):
    def testConstants(self):
        self.assertIsInstance(WebKit.WebHistoryItemChangedNotification, str)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKUserScript(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(WebKit.WKUserScriptInjectionTimeAtDocumentStart, 0)
        self.assertEqual(WebKit.WKUserScriptInjectionTimeAtDocumentEnd, 1)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(WebKit.WKUserScript.isForMainFrameOnly)
        self.assertArgIsBOOL(
            WebKit.WKUserScript.initWithSource_injectionTime_forMainFrameOnly_, 2
        )

    @min_os_level("10.16")
    def testMethods10_16(self):
        self.assertArgIsBOOL(
            WebKit.WKUserScript.initWithSource_injectionTime_forMainFrameOnly_inContentWorld_,
            2,
        )



from PyObjCTools.TestSupport import TestCase
import WebKit  # noqa: F401
import objc


class TestWebDocumentHelper(WebKit.NSObject):
    def setNeedsLayout_(self, v):
        pass

    def searchFor_direction_caseSensitive_wrap_(self, a, b, c, d):
        return 1

    def supportsTextEncoding(self):
        return 1

    def canProvideDocumentSource(self):
        return 1


class TestWebDocument(TestCase):
    def testProtocols(self):
        self.assertIsInstance(
            objc.protocolNamed("WebDocumentView"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("WebDocumentSearching"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("WebDocumentText"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("WebDocumentRepresentation"), objc.formal_protocol
        )

    def testMethods(self):
        self.assertArgIsBOOL(TestWebDocumentHelper.setNeedsLayout_, 0)

        self.assertResultIsBOOL(
            TestWebDocumentHelper.searchFor_direction_caseSensitive_wrap_
        )
        self.assertArgIsBOOL(
            TestWebDocumentHelper.searchFor_direction_caseSensitive_wrap_, 1
        )
        self.assertArgIsBOOL(
            TestWebDocumentHelper.searchFor_direction_caseSensitive_wrap_, 2
        )
        self.assertArgIsBOOL(
            TestWebDocumentHelper.searchFor_direction_caseSensitive_wrap_, 3
        )

        self.assertResultIsBOOL(TestWebDocumentHelper.supportsTextEncoding)
        self.assertResultIsBOOL(TestWebDocumentHelper.canProvideDocumentSource)



from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import WebKit
import objc


class TestWebPluginViewFactory(TestCase):
    def testConstants(self):
        self.assertIsInstance(WebKit.WebPlugInBaseURLKey, str)
        self.assertIsInstance(WebKit.WebPlugInAttributesKey, str)
        self.assertIsInstance(WebKit.WebPlugInContainerKey, str)
        self.assertIsInstance(WebKit.WebPlugInContainingElementKey, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(WebKit.WebPlugInShouldLoadMainResourceKey, str)

    @min_sdk_level("10.6")
    def testProtocols(self):
        objc.protocolNamed("WebPlugInViewFactory")



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMMutationEvent(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_MODIFICATION, 1)
        self.assertEqual(WebKit.DOM_ADDITION, 2)
        self.assertEqual(WebKit.DOM_REMOVAL, 3)

    def testMethods(self):
        self.assertArgIsBOOL(
            WebKit.DOMMutationEvent.initMutationEvent_canBubble_cancelable_relatedNode_prevValue_newValue_attrName_attrChange_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            WebKit.DOMMutationEvent.initMutationEvent_canBubble_cancelable_relatedNode_prevValue_newValue_attrName_attrChange_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(WebKit.DOMMutationEvent.initMutationEvent________, 2)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKError(TestCase):
    @min_os_level("10.13")
    def test_methods(self):
        self.assertArgIsBlock(WebKit.WKHTTPCookieStore.getAllCookies_, 0, b"v@")
        self.assertArgIsBlock(
            WebKit.WKHTTPCookieStore.setCookie_completionHandler_, 1, b"v"
        )
        self.assertArgIsBlock(
            WebKit.WKHTTPCookieStore.deleteCookie_completionHandler_, 1, b"v"
        )



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestWebJavaPlugIn(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.WebJNIReturnTypeInvalid, 0)
        self.assertEqual(WebKit.WebJNIReturnTypeVoid, 1)
        self.assertEqual(WebKit.WebJNIReturnTypeObject, 2)
        self.assertEqual(WebKit.WebJNIReturnTypeBoolean, 3)
        self.assertEqual(WebKit.WebJNIReturnTypeByte, 4)
        self.assertEqual(WebKit.WebJNIReturnTypeChar, 5)
        self.assertEqual(WebKit.WebJNIReturnTypeShort, 6)
        self.assertEqual(WebKit.WebJNIReturnTypeInt, 7)
        self.assertEqual(WebKit.WebJNIReturnTypeLong, 8)
        self.assertEqual(WebKit.WebJNIReturnTypeFloat, 9)
        self.assertEqual(WebKit.WebJNIReturnTypeDouble, 10)



from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level, cast_uint
import WebKit
import objc


class TestDOMNodeFilterHelper(WebKit.NSObject):
    def acceptNode_(self, n):
        return 1


class TestDOMNodeFilter(TestCase):
    @min_sdk_level("10.11")
    def testProtocols(self):
        objc.protocolNamed("DOMNodeFilter")

    def testMethods(self):
        self.assertResultHasType(TestDOMNodeFilterHelper.acceptNode_, objc._C_SHT)
        self.assertResultIsBOOL(WebKit.DOMNodeIterator.expandEntityReferences)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(WebKit.DOMNodeIterator.pointerBeforeReferenceNode)

    def testConstants(self):
        self.assertEqual(WebKit.DOM_FILTER_ACCEPT, 1)
        self.assertEqual(WebKit.DOM_FILTER_REJECT, 2)
        self.assertEqual(WebKit.DOM_FILTER_SKIP, 3)
        self.assertEqual(WebKit.DOM_SHOW_ALL, cast_uint(0xFFFFFFFF))
        self.assertEqual(WebKit.DOM_SHOW_ELEMENT, 0x00000001)
        self.assertEqual(WebKit.DOM_SHOW_ATTRIBUTE, 0x00000002)
        self.assertEqual(WebKit.DOM_SHOW_TEXT, 0x00000004)
        self.assertEqual(WebKit.DOM_SHOW_CDATA_SECTION, 0x00000008)
        self.assertEqual(WebKit.DOM_SHOW_ENTITY_REFERENCE, 0x00000010)
        self.assertEqual(WebKit.DOM_SHOW_ENTITY, 0x00000020)
        self.assertEqual(WebKit.DOM_SHOW_PROCESSING_INSTRUCTION, 0x00000040)
        self.assertEqual(WebKit.DOM_SHOW_COMMENT, 0x00000080)
        self.assertEqual(WebKit.DOM_SHOW_DOCUMENT, 0x00000100)
        self.assertEqual(WebKit.DOM_SHOW_DOCUMENT_TYPE, 0x00000200)
        self.assertEqual(WebKit.DOM_SHOW_DOCUMENT_FRAGMENT, 0x00000400)
        self.assertEqual(WebKit.DOM_SHOW_NOTATION, 0x00000800)



import contextlib

import JavaScriptCore


@contextlib.contextmanager
def autoreleasing(value):
    try:
        yield value

    finally:
        if isinstance(value, JavaScriptCore.JSContextGroupRef):
            JavaScriptCore.JSContextGroupRelease(value)

        elif isinstance(value, JavaScriptCore.JSGlobalContextRef):
            JavaScriptCore.JSGlobalContextRelease(value)

        elif isinstance(value, JavaScriptCore.JSClassRef):
            JavaScriptCore.JSClassRelease(value)

        elif isinstance(value, JavaScriptCore.JSPropertyNameArrayRef):
            JavaScriptCore.JSPropertyNameArrayRelease(value)

        elif isinstance(value, JavaScriptCore.JSStringRef):
            JavaScriptCore.JSStringRelease(value)



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc
import WebKit


class TestWKScriptMessageHandlerWithReplyHelper(WebKit.NSObject):
    def userContentController_didReceiveScriptMessage_replyHandler_(self, a, b, c):
        pass


class TestWKScriptMessageHandlerWithReply(TestCase):
    @min_sdk_level("10.16")
    def testProtocols10_16(self):
        objc.protocolNamed("WKScriptMessageHandlerWithReply")

    def testMethods(self):
        self.assertArgIsBlock(
            TestWKScriptMessageHandlerWithReplyHelper.userContentController_didReceiveScriptMessage_replyHandler_,
            2,
            b"v@@",
        )



import JavaScriptCore
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestJSContextRef(TestCase):
    def test_functions(self):
        self.assertResultHasType(
            JavaScriptCore.JSContextGroupCreate,
            JavaScriptCore.JSContextGroupRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSContextGroupRetain,
            JavaScriptCore.JSContextGroupRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSContextGroupRetain,
            0,
            JavaScriptCore.JSContextGroupRef.__typestr__,
        )

        self.assertArgHasType(
            JavaScriptCore.JSContextGroupRelease,
            0,
            JavaScriptCore.JSContextGroupRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSGlobalContextCreate,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSGlobalContextCreate,
            0,
            JavaScriptCore.JSClassRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSGlobalContextCreateInGroup,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSGlobalContextCreateInGroup,
            0,
            JavaScriptCore.JSContextGroupRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSGlobalContextCreateInGroup,
            1,
            JavaScriptCore.JSClassRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSGlobalContextRetain,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSGlobalContextRetain,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )

        self.assertArgHasType(
            JavaScriptCore.JSGlobalContextRelease,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSContextGetGlobalObject,
            JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSContextGetGlobalObject,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSContextGetGroup,
            JavaScriptCore.JSContextGroupRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSContextGetGroup, 0, JavaScriptCore.JSContextRef.__typestr__
        )

    @min_os_level("10.7")
    def test_functions10_7(self):
        self.assertResultHasType(
            JavaScriptCore.JSContextGetGlobalContext,
            JavaScriptCore.JSGlobalContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSContextGetGlobalContext,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )

    @min_os_level("10.10")
    def test_functions10_10(self):
        self.assertResultHasType(
            JavaScriptCore.JSGlobalContextCopyName,
            JavaScriptCore.JSStringRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSGlobalContextCopyName,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )

        self.assertResultHasType(JavaScriptCore.JSGlobalContextSetName, objc._C_VOID)
        self.assertArgHasType(
            JavaScriptCore.JSGlobalContextSetName,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSGlobalContextSetName,
            1,
            JavaScriptCore.JSStringRef.__typestr__,
        )



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMWheelEvent(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMWheelEvent.ctrlKey)
        self.assertResultIsBOOL(WebKit.DOMWheelEvent.shiftKey)
        self.assertResultIsBOOL(WebKit.DOMWheelEvent.altKey)
        self.assertResultIsBOOL(WebKit.DOMWheelEvent.metaKey)
        self.assertResultIsBOOL(WebKit.DOMWheelEvent.isHorizontal)

        self.assertArgIsBOOL(
            WebKit.DOMWheelEvent.initWheelEvent_wheelDeltaY_view_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            7,
        )
        self.assertArgIsBOOL(
            WebKit.DOMWheelEvent.initWheelEvent_wheelDeltaY_view_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            8,
        )
        self.assertArgIsBOOL(
            WebKit.DOMWheelEvent.initWheelEvent_wheelDeltaY_view_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            9,
        )
        self.assertArgIsBOOL(
            WebKit.DOMWheelEvent.initWheelEvent_wheelDeltaY_view_screenX_screenY_clientX_clientY_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            10,
        )

    def testConstants(self):
        self.assertEqual(WebKit.DOM_DOM_DELTA_PIXEL, 0)
        self.assertEqual(WebKit.DOM_DOM_DELTA_LINE, 1)
        self.assertEqual(WebKit.DOM_DOM_DELTA_PAGE, 2)
